import { BaseAction, IActionPayload } from './BaseAction';
import { Meta, Metas, MetasLoading, IMetaAction } from '../models/MetaModel';
import { State, StateReturn } from '../models/BaseModel';
import { HTTP_STATUS_CODE } from '../utils/httpStatusCode';
import { HttpServiceBuilder } from '../services/HttpServiceBuilder';
import { METHOD } from '../utils/method';
export interface Types {
    prepare: string;
    success: string;
    fail: string;
}
export interface IBaseRequestAction<Data = any, Response = any, Payload = any, Type = Types> extends IActionPayload<Payload, Type>, IMetaAction {
    uniqueId: number;
    method: METHOD;
    uri: string;
    body: object;
    query: object;
    successText: string;
    failText: string;
    hideError: boolean | ((response: IResponseAction<Response, Payload>) => boolean);
    requestOptions: object;
    modelName: string;
    useThrottle: boolean;
    throttleMillSeconds: number;
    throttleKey: string;
    throttleDeps: any[];
    onPrepare: null | ((state: State<Data>, action: IActionPayload<Payload>) => StateReturn<Data>);
    onSuccess: null | ((state: State<Data>, action: IResponseAction<Response, Payload>) => StateReturn<Data>);
    onFail: null | ((state: State<Data>, action: IResponseAction<unknown, Payload>) => StateReturn<Data>);
}
export interface HttpTransform {
    httpStatus?: HTTP_STATUS_CODE;
    message?: string;
    businessCode?: string;
}
export interface IResponseAction<Response = any, Payload = any> extends IActionPayload<Payload>, HttpTransform {
    response: Response;
}
export interface InternalPrepareAction<Data = any, Response = any, Payload = any> extends IBaseRequestAction<Data, Response, Payload, string> {
    loading: boolean;
    effect: IBaseRequestAction<Data, any, Payload>['onPrepare'];
}
export interface InternalSuccessAction<Data = any, Response = any, Payload = any> extends IBaseRequestAction<Data, Response, Payload, string>, IResponseAction<Response, Payload> {
    loading: boolean;
    effect: IBaseRequestAction<Data, Response, Payload, string>['onSuccess'];
    fromThrottle?: boolean;
}
export interface RequestSubscriber<CustomData, Response, Payload> {
    when: string;
    effect: (state: State<CustomData>, action: IResponseAction<Response, Payload>) => StateReturn<CustomData>;
}
export declare const requestActionProxyKeys: {
    methods: (keyof BaseRequestAction<any, any, any, any, any>)[];
    getters: (keyof BaseRequestAction<any, any, any, any, any>)[];
};
export declare class BaseRequestAction<Data, Builder extends (...args: any[]) => HttpServiceBuilder<Data, Response, Payload, any, M>, Response, Payload, M> extends BaseAction<Data> {
    protected readonly builder: Builder;
    protected loadingsCache?: [Metas, MetasLoading<any>];
    protected readonly uniqueId: number;
    private __prepareType?;
    private __failType?;
    constructor(builder: Builder, uniqueId: number, fromSubClass?: boolean);
    clearThrottle(): void;
    get meta(): Meta;
    get loading(): boolean;
    get metas(): Metas;
    get loadings(): MetasLoading<any>;
    getPrepareType(): string;
    getFailType(): string;
    onSuccess<CustomData>(effect: RequestSubscriber<CustomData, Response, Payload>['effect']): RequestSubscriber<CustomData, Response, Payload>;
    onPrepare<CustomData>(effect: RequestSubscriber<CustomData, Response, Payload>['effect']): RequestSubscriber<CustomData, Response, Payload>;
    onFail<CustomData>(effect: RequestSubscriber<CustomData, Response, Payload>['effect']): RequestSubscriber<CustomData, Response, Payload>;
    protected getLoadingHandler(metas: Metas): MetasLoading<any>;
    /**
     * @override
     */
    protected getProxyMethods(): string[];
    /**
     * @override
     */
    protected getProxyGetters(): string[];
    /**
     * @implements
     */
    protected getProxyFn(): Function;
    /**
     * @override
     */
    protected setName(name: string | number): void;
}
