"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRequestAction = exports.requestActionProxyKeys = void 0;
var tslib_1 = require("tslib");
var BaseAction_1 = require("./BaseAction");
var setModel_1 = require("../utils/setModel");
var MetaModel_1 = require("../models/MetaModel");
var setActionName_1 = require("../utils/setActionName");
var BaseHttpService_1 = require("../services/BaseHttpService");
var StoreHelper_1 = require("../stores/StoreHelper");
exports.requestActionProxyKeys = {
    methods: tslib_1.__spread(['onSuccess', 'onPrepare', 'onFail', 'getPrepareType', 'getFailType'], BaseAction_1.actionProxyKeys.methods),
    getters: tslib_1.__spread([
        'meta', 'metas',
        'loading', 'loadings'
    ], BaseAction_1.actionProxyKeys.getters),
};
var BaseRequestAction = /** @class */ (function (_super) {
    tslib_1.__extends(BaseRequestAction, _super);
    function BaseRequestAction(builder, uniqueId, fromSubClass) {
        if (fromSubClass === void 0) { fromSubClass = false; }
        var _this = _super.call(this, setModel_1.getCurrentModel()) || this;
        _this.builder = builder;
        _this.uniqueId = uniqueId;
        return fromSubClass ? _this : _this.proxy();
    }
    BaseRequestAction.prototype.clearThrottle = function () {
        StoreHelper_1.storeHelper.dispatch({
            type: BaseHttpService_1.CLEAR_THROTTLE,
            key: this.getSuccessType(),
            uniqueId: this.uniqueId,
        });
    };
    Object.defineProperty(BaseRequestAction.prototype, "meta", {
        get: function () {
            return MetaModel_1.metaModel.getMeta(this.instanceName) || MetaModel_1.DEFAULT_META;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "loading", {
        get: function () {
            return this.meta.loading;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "metas", {
        get: function () {
            return MetaModel_1.metaModel.getMeta(this.instanceName) || MetaModel_1.DEFAULT_METAS;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "loadings", {
        get: function () {
            return this.getLoadingHandler(this.metas);
        },
        enumerable: false,
        configurable: true
    });
    BaseRequestAction.prototype.getPrepareType = function () {
        return this.__prepareType || setActionName_1.setActionName(this).__prepareType;
    };
    BaseRequestAction.prototype.getFailType = function () {
        return this.__failType || setActionName_1.setActionName(this).__failType;
    };
    BaseRequestAction.prototype.onSuccess = function (effect) {
        return {
            when: this.getSuccessType(),
            effect: effect,
        };
    };
    BaseRequestAction.prototype.onPrepare = function (effect) {
        return {
            when: this.getPrepareType(),
            effect: effect,
        };
    };
    BaseRequestAction.prototype.onFail = function (effect) {
        return {
            when: this.getFailType(),
            effect: effect,
        };
    };
    BaseRequestAction.prototype.getLoadingHandler = function (metas) {
        if (!this.loadingsCache || this.loadingsCache[0] !== metas) {
            this.loadingsCache = [metas, {
                    pick: function (payload) { return metas.pick(payload).loading; },
                }];
        }
        return this.loadingsCache[1];
    };
    /**
     * @override
     */
    BaseRequestAction.prototype.getProxyMethods = function () {
        return exports.requestActionProxyKeys.methods;
    };
    /**
     * @override
     */
    BaseRequestAction.prototype.getProxyGetters = function () {
        return exports.requestActionProxyKeys.getters;
    };
    /**
     * @implements
     */
    BaseRequestAction.prototype.getProxyFn = function () {
        var _this = this;
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return StoreHelper_1.storeHelper.dispatch(_this.builder.apply(_this, tslib_1.__spread(args)).collect(_this));
        };
    };
    /**
     * @override
     */
    BaseRequestAction.prototype.setName = function (name) {
        _super.prototype.setName.call(this, name);
        this.__prepareType = this.instanceName + ' prepare';
        this.__failType = this.instanceName + ' fail';
    };
    return BaseRequestAction;
}(BaseAction_1.BaseAction));
exports.BaseRequestAction = BaseRequestAction;
