"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseHttpService = exports.CLEAR_THROTTLE = void 0;
var tslib_1 = require("tslib");
var BaseRequestAction_1 = require("../actions/BaseRequestAction");
var clone_1 = tslib_1.__importDefault(require("clone"));
var OrphanHttpService_1 = require("./OrphanHttpService");
var method_1 = require("../utils/method");
var StoreHelper_1 = require("../stores/StoreHelper");
exports.CLEAR_THROTTLE = '@flus/clear/throttle';
var BaseHttpService = /** @class */ (function () {
    function BaseHttpService(config) {
        this.caches = {};
        this.config = config;
        this.uniqueId = Date.now() + Math.random();
        StoreHelper_1.storeHelper.middleware.use(this.uniqueId + '_throttle', this.createThrottleMiddleware());
        StoreHelper_1.storeHelper.middleware.use(this.uniqueId + '_request', this.createRequestMiddleware());
    }
    BaseHttpService.prototype.action = function (fn) {
        return new BaseRequestAction_1.BaseRequestAction(fn, this.uniqueId);
    };
    BaseHttpService.prototype.getAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.get, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.postAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.post, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.putAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.put, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.deleteAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.delete, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.patchAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.patch, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.connectAsync = function (config) {
        var service = new OrphanHttpService_1.OrphanHttpService(config, method_1.METHOD.connect, this.uniqueId);
        return StoreHelper_1.storeHelper.dispatch(service.collect());
    };
    BaseHttpService.prototype.generateThrottleKey = function (action) {
        return JSON.stringify([
            action.modelName,
            action.type,
            action.uri,
            action.method,
            action.body,
            action.query,
            action.requestOptions,
        ]);
    };
    BaseHttpService.prototype.isFetchAction = function (action) {
        return typeof action.type === 'object';
    };
    BaseHttpService.prototype.isClearAction = function (action) {
        return action.type === exports.CLEAR_THROTTLE;
    };
    BaseHttpService.prototype.createThrottleMiddleware = function () {
        var _this = this;
        return function () { return function (next) { return function (action) {
            if (action.uniqueId !== _this.uniqueId) {
                return next(action);
            }
            if (_this.isClearAction(action)) {
                _this.caches[action.key] === undefined;
                return action;
            }
            if (_this.isFetchAction(action)) {
                var key = action.type.success;
                var cacheData = _this.caches[key];
                if (!action.useThrottle) {
                    if (cacheData) {
                        cacheData[_this.generateThrottleKey(action)] = undefined;
                    }
                    return next(action);
                } // end
                var throttleKey = _this.generateThrottleKey(action);
                var item_1 = cacheData === null || cacheData === void 0 ? void 0 : cacheData[throttleKey];
                action.throttleKey = throttleKey;
                if (item_1 && Date.now() <= item_1.timestamp) {
                    var promise = new Promise(function (resolve) {
                        var fakeOkAction = tslib_1.__assign(tslib_1.__assign({}, action), { loading: false, type: action.type.success, response: clone_1.default(item_1.response, false), effect: action.onSuccess, fromThrottle: true });
                        StoreHelper_1.storeHelper.dispatch(fakeOkAction);
                        _this.triggerShowSuccess(fakeOkAction, action.successText);
                        resolve(fakeOkAction);
                    });
                    var wrapPromise = promise;
                    wrapPromise.cancel = function () { };
                    return wrapPromise;
                } // end
                if (item_1) {
                    cacheData[throttleKey] = undefined;
                }
                return next(action);
            } // end
            if (action.useThrottle && !action.fromThrottle && action.throttleMillSeconds > 0) {
                var type = action.type;
                if (type.lastIndexOf('success') !== type.length - 7) {
                    return next(action);
                } // end
                _this.caches[type] = _this.caches[type] || {};
                _this.caches[type][action.throttleKey] = {
                    timestamp: Date.now() + action.throttleMillSeconds,
                    response: clone_1.default(action.response, false),
                };
            }
            return next(action);
        }; }; };
    };
    BaseHttpService.prototype.createRequestMiddleware = function () {
        var _this = this;
        return function () { return function (next) { return function (action) {
            if (action.uniqueId !== _this.uniqueId || typeof action.type !== 'object') {
                return next(action);
            }
            return _this.runAction(action);
        }; }; };
    };
    BaseHttpService.prototype.triggerShowSuccess = function (okResponse, successText) {
        if (successText) {
            this.config.onShowSuccess(successText, okResponse);
        }
    };
    BaseHttpService.prototype.triggerShowError = function (errorResponse, hideError) {
        if (!errorResponse.message) {
            return;
        }
        var showError;
        if (typeof hideError === 'boolean') {
            showError = !hideError;
        }
        else {
            showError = !hideError(errorResponse);
        }
        if (showError) {
            this.config.onShowError(errorResponse.message, errorResponse);
        }
    };
    return BaseHttpService;
}());
exports.BaseHttpService = BaseHttpService;
