import { Store, PreloadedState, Reducer, AnyAction, Middleware, compose } from 'redux';
import { IReducers } from '../reducers/BaseReducer';
import { PersistStorage } from './PersistStorage';
import { BaseModel } from '../models/BaseModel';
import { Persist } from './Persist';
import { DynamicMiddleware } from './DynamicMiddleware';
export interface ReduxStoreConfig<Engine extends string = 'memory'> {
    reducers?: IReducers;
    compose?: typeof compose;
    middleware?: Middleware[];
    preloadedState?: PreloadedState<any>;
    onCombineReducers?: (reducer: Reducer) => Reducer;
    persist?: false | {
        version: string | number;
        key: string;
        storage: PersistStorage | Engine;
        allowlist: Record<string, BaseModel<any>>;
    };
}
export declare class StoreHelper {
    protected readonly __persist: Persist;
    protected readonly __dynamicMiddleware: DynamicMiddleware;
    protected __store?: Store;
    protected autoReducers: IReducers;
    protected userReducers: IReducers;
    protected listeners: Array<(storeHelper: StoreHelper) => void>;
    protected isDispatching: boolean;
    protected stateWhenDispatching: object;
    protected onCombineReducers: ReduxStoreConfig['onCombineReducers'];
    constructor();
    createStore(config: ReduxStoreConfig): Store;
    appendReducers(reducers: IReducers): this;
    get middleware(): DynamicMiddleware;
    get store(): Store;
    get persist(): Persist;
    dispatch<T extends AnyAction>(action: T): any;
    getState(): object;
    listenOnce(fn: (storeHelper: StoreHelper) => void): this;
    protected publish(): void;
    protected combindReducers(): Reducer;
}
export declare const storeHelper: StoreHelper;
