"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.storeHelper = exports.StoreHelper = void 0;
var tslib_1 = require("tslib");
var redux_1 = require("redux");
var StoreNotFoundError_1 = require("../exceptions/StoreNotFoundError");
var Persist_1 = require("./Persist");
var DynamicMiddleware_1 = require("./DynamicMiddleware");
var StoreHelper = /** @class */ (function () {
    function StoreHelper() {
        this.autoReducers = {};
        this.userReducers = {};
        this.listeners = [];
        this.isDispatching = false;
        this.stateWhenDispatching = {};
        this.__persist = new Persist_1.Persist(this);
        this.__dynamicMiddleware = new DynamicMiddleware_1.DynamicMiddleware();
    }
    StoreHelper.prototype.createStore = function (config) {
        var onCombineReducers = config.onCombineReducers, _a = config.reducers, reducers = _a === void 0 ? {} : _a, preloadedState = config.preloadedState, _b = config.compose, customCompose = _b === void 0 ? redux_1.compose : _b, _c = config.middleware, middleware = _c === void 0 ? [] : _c;
        this.onCombineReducers = onCombineReducers;
        this.userReducers = reducers;
        this.__persist.setConfig(config.persist);
        var combined = this.combindReducers();
        if (this.__store) {
            this.store.replaceReducer(combined);
        }
        else {
            this.__store = redux_1.createStore(combined, preloadedState, customCompose(redux_1.applyMiddleware.apply(void 0, tslib_1.__spread([this.__dynamicMiddleware.create()], middleware))));
            this.publish();
        }
        return this.__store;
    };
    StoreHelper.prototype.appendReducers = function (reducers) {
        this.autoReducers = tslib_1.__assign(tslib_1.__assign({}, this.autoReducers), reducers);
        if (this.__store) {
            this.__store.replaceReducer(this.combindReducers());
        }
        return this;
    };
    Object.defineProperty(StoreHelper.prototype, "middleware", {
        get: function () {
            return this.__dynamicMiddleware;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(StoreHelper.prototype, "store", {
        get: function () {
            if (!this.__store) {
                throw new StoreNotFoundError_1.StoreNotFoundError();
            }
            return this.__store;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(StoreHelper.prototype, "persist", {
        get: function () {
            return this.__persist;
        },
        enumerable: false,
        configurable: true
    });
    StoreHelper.prototype.dispatch = function (action) {
        return this.store.dispatch(action);
    };
    StoreHelper.prototype.getState = function () {
        return this.isDispatching ? this.stateWhenDispatching : this.store.getState();
    };
    StoreHelper.prototype.listenOnce = function (fn) {
        var _this = this;
        this.listeners.push(fn);
        if (this.__store) {
            setTimeout(function () {
                _this.publish();
            });
        }
        return this;
    };
    StoreHelper.prototype.publish = function () {
        var _this = this;
        this.listeners.forEach(function (listener) { return listener(_this); });
        this.listeners = [];
    };
    StoreHelper.prototype.combindReducers = function () {
        var _this = this;
        var combined = redux_1.combineReducers(tslib_1.__assign(tslib_1.__assign({}, this.autoReducers), this.userReducers));
        if (this.onCombineReducers) {
            combined = this.onCombineReducers(combined);
        }
        return function (state, action) {
            _this.isDispatching = true;
            _this.stateWhenDispatching = state;
            var mainResult = combined(state, action);
            if (_this.stateWhenDispatching !== mainResult) {
                _this.persist.update(mainResult, action.type === Persist_1.TYPE_REHYDRATE);
            }
            _this.isDispatching = false;
            _this.stateWhenDispatching = {};
            return mainResult;
        };
    };
    return StoreHelper;
}());
exports.StoreHelper = StoreHelper;
exports.storeHelper = new StoreHelper();
