import { AxiosRequestConfig, AxiosResponse, Canceler, AxiosInstance } from 'axios';
import { BaseHttpService, HttpServiceBuilderWithMeta, PickPayload, PickResponse, HttpServiceBuilderWithMetas, PickData, PickMeta, IBaseRequestAction, BaseHttpServiceConfig, HttpTransform, FetchHandle as SuperFetchHandle } from '../core';
import { RequestAction } from '../actions/RequestAction';
export declare type HttpResponse<T = any> = AxiosResponse<T>;
export declare type HttpCanceler = Canceler;
export interface FetchHandle<Response = any, Payload = any> extends SuperFetchHandle<Response, Payload, HttpCanceler> {
}
export interface HttpServiceConfig extends BaseHttpServiceConfig {
    requestConfig?: AxiosRequestConfig;
    onRespondError: (httpResponse: HttpResponse, transform: HttpTransform) => void;
    headers: (action: IRequestAction) => object;
    beforeSend?: (action: IRequestAction) => void;
    isSuccess?: (response: HttpResponse) => boolean;
    transformSuccessData?: (data: any, headers: any) => any;
}
export interface IRequestAction<Data = any, Response = any, Payload = any> extends IBaseRequestAction<Data, Response, Payload> {
    requestOptions: AxiosRequestConfig;
}
export declare class HttpService extends BaseHttpService<HttpServiceConfig, HttpCanceler> {
    protected readonly httpHandler: AxiosInstance;
    constructor(config: HttpServiceConfig);
    clone(config: Partial<HttpServiceConfig>): HttpService;
    action<Fn extends (...args: any[]) => HttpServiceBuilderWithMeta<Data, Response, Payload, AxiosRequestConfig>, Data = PickData<Fn>, Response = PickResponse<Fn>, Payload = PickPayload<Fn>>(fn: Fn): ((...args: Parameters<Fn>) => FetchHandle<Response, Payload>) & Omit<RequestAction<Data, Fn, Response, Payload, true>, 'metas' | 'loadings' | 'useMetas' | 'useLoadings'>;
    action<Fn extends (...args: any[]) => HttpServiceBuilderWithMetas<Data, Response, Payload, AxiosRequestConfig, M>, Data = PickData<Fn>, Response = PickResponse<Fn>, Payload = PickPayload<Fn>, M = PickMeta<Fn>>(fn: Fn): ((...args: Parameters<Fn>) => FetchHandle<Response, Payload>) & Omit<RequestAction<Data, Fn, Response, Payload, M>, 'meta' | 'loading' | 'useMeta' | 'useLoading'>;
    protected runAction(action: IRequestAction): Promise<any>;
}
